@@ -13,8 +13,9 @@
 
 """This file includes the python code of CADC.rst."""
 
-"""Authorize a Client Object"""
+"""Authorize a Client Instance"""
 from tensorbay import GAS
+from tensorbay.dataset import FusionDataset
 
 ACCESS_KEY = "Accesskey-*****"
 gas = GAS(ACCESS_KEY)
@@ -25,7 +26,7 @@ gas.create_dataset("CADC", is_fusion=True)
 """"""
 
 """List Dataset Names"""
-list(gas.list_dataset_names())
+gas.list_dataset_names()
 """"""
 
 from tensorbay.opendataset import CADC
@@ -35,21 +36,19 @@ fusion_dataset = CADC("path/to/dataset/directory")
 """Upload Fusion Dataset"""
 # fusion_dataset is the one you initialized in "Organize Fusion Dataset" section
 fusion_dataset_client = gas.upload_dataset(fusion_dataset, jobs=8, skip_uploaded_files=False)
-fusion_dataset_client.commit("CADC")
+fusion_dataset_client.commit("initial commit")
 """"""
 
 """Read Fusion Dataset / get fusion dataset"""
-fusion_dataset_client = gas.get_dataset("CADC", is_fusion=True)
+fusion_dataset = FusionDataset("CADC", gas)
 """"""
 
-"""Read Dataset / list fusion segment names"""
-list(fusion_dataset_client.list_segment_names())
+"""Read Fusion Dataset / list fusion segment names"""
+fusion_dataset.keys()
 """"""
 
 """Read Fusion Dataset / get fusion segment"""
-from tensorbay.dataset import FusionSegment
-
-fusion_segment = FusionSegment("2018_03_06/0001", fusion_dataset_client)
+fusion_segment = fusion_dataset["2018_03_06/0001"]
 """"""
 
 """Read Fusion Dataset / get sensors"""
