@@ -10,10 +10,12 @@
 # pylint: disable=import-error
 # pylint: disable=pointless-string-statement
 # pylint: disable=invalid-name
+# pylint: disable=unused-import
+# flake8: noqa: F401
 
 """This file includes the python code of NeolixOD.rst."""
 
-"""Authorize a Client Object"""
+"""Authorize a Client Instance"""
 from tensorbay import GAS
 
 ACCESS_KEY = "Accesskey-*****"
@@ -21,35 +23,36 @@ gas = GAS(ACCESS_KEY)
 """"""
 
 """Create Dataset"""
-gas.create_dataset("Neolix OD")
+gas.create_dataset("NeolixOD")
 """"""
 
-"""List Dataset Names"""
-list(gas.list_dataset_names())
+"""Organize Dataset / regular import"""
+from tensorbay.dataset import Data, Dataset
+from tensorbay.label import LabeledBox3D
+
 """"""
 
+"""Organize dataset / import dataloader"""
 from tensorbay.opendataset import NeolixOD
 
 dataset = NeolixOD("path/to/dataset/directory")
+""""""
 
 """Upload Dataset"""
-# dataset is the one you initialized in "Organize Dataset" section
-dataset_client = gas.upload_dataset(dataset, jobs=8, skip_uploaded_files=False)
-dataset_client.commit("Neolix OD")
+dataset_client = gas.upload_dataset(dataset)
+dataset_client.commit("initial commit")
 """"""
 
 """Read Dataset / get dataset"""
-dataset_client = gas.get_dataset("Neolix OD")
+dataset = Dataset("NeolixOD", gas)
 """"""
 
 """Read Dataset / get segment"""
-from tensorbay.dataset import Segment
-
-default_segment = Segment("", dataset_client)
+segment = dataset[0]
 """"""
 
 """Read Dataset / get data"""
-data = default_segment[0]
+data = segment[0]
 """"""
 
 """Read Dataset / get label"""
@@ -59,5 +62,5 @@ attributes = label_box3d.attributes
 """"""
 
 """Delete Dataset"""
-gas.delete_dataset("Neolix OD")
+gas.delete_dataset("NeolixOD")
 """"""
