@@ -12,7 +12,7 @@
 
 """This file includes the python code of getting_started_with_tensorbay.rst."""
 
-"""Authorize a Client Object"""
+"""Authorize a Client Instance"""
 from tensorbay import GAS
 
 gas = GAS("<YOUR_ACCESSKEY>")
@@ -23,7 +23,7 @@ gas.create_dataset("DatasetName")
 """"""
 
 """List Dataset Names"""
-dataset_list = list(gas.list_dataset_names())
+dataset_names = gas.list_dataset_names()
 """"""
 
 """Upload Images to the Dataset"""
@@ -46,11 +46,9 @@ dataset_client.commit("Initial commit")
 
 """Read Images from the Dataset"""
 from PIL import Image
-from tensorbay.dataset import Segment
 
-dataset_client = gas.get_dataset("DatasetName")
-
-segment = Segment("", dataset_client)
+dataset = Dataset("DatasetName", gas)
+segment = dataset[0]
 
 for data in segment:
     with data.open() as fp:
