@@ -26,11 +26,10 @@ import sys
 from typing import TYPE_CHECKING, Any, Dict, Generator, Iterable, Iterator, Optional, Tuple, Union
 
 from ..dataset import Data, Frame, FusionSegment, Notes, Segment
-from ..exception import FrameError
+from ..exception import FrameError, NameConflictError, ResourceNotExistError
 from ..label import Catalog
 from ..utility import Deprecated, KwargsDeprecated
 from .commit_status import CommitStatus
-from .exceptions import GASSegmentError
 from .requests import Client, PagingList, Tqdm, multithread_upload
 from .segment import FusionSegmentClient, SegmentClient
 from .struct import Branch, Commit, Draft, Tag
@@ -71,7 +70,7 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
 
         self._status = CommitStatus()
         if commit_id:
-            self.checkout(revision=commit_id)
+            self._status.checkout(commit_id=commit_id)
 
     def _commit(self, message: str, tag: Optional[str] = None) -> str:
         post_data: Dict[str, Any] = {
@@ -153,16 +152,18 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
 
         self._client.open_api_do("POST", "segments", self.dataset_id, json=post_data)
 
-    def _generate_segment_names(
-        self, offset: int = 0, limit: int = 128
-    ) -> Generator[str, None, int]:
+    def _list_segments(self, offset: int = 0, limit: int = 128) -> Dict[str, Any]:
         params: Dict[str, Any] = self._status.get_status_info()
         params["offset"] = offset
         params["limit"] = limit
 
-        response = self._client.open_api_do(
-            "GET", "segments", self.dataset_id, params=params
-        ).json()
+        response = self._client.open_api_do("GET", "segments", self._dataset_id, params=params)
+        return response.json()  # type: ignore[no-any-return]
+
+    def _generate_segment_names(
+        self, offset: int = 0, limit: int = 128
+    ) -> Generator[str, None, int]:
+        response = self._list_segments(offset, limit)
 
         for item in response["segments"]:
             yield item["name"]
@@ -225,7 +226,8 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
             The :class:`.Draft` instance with the given number.
 
         Raises:
-            TypeError: When the required draft does not exist or the given draft number is illegal.
+            TypeError: When the given draft number is illegal.
+            ResourceNotExistError: When the required draft does not exist.
 
         """
         if draft_number is None:
@@ -239,7 +241,7 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
             if draft_number == draft.number:
                 return draft
 
-        raise TypeError(f"The draft: {draft_number} does not exist.")
+        raise ResourceNotExistError(resource="draft", identification=draft_number)
 
     @KwargsDeprecated(
         ("start", "stop"),
@@ -257,10 +259,14 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
             The PagingList of :class:`drafts<.Draft>`.
 
         """
-        start = kwargs.get("start", 0)
-        stop = kwargs.get("stop", sys.maxsize)
+        start = kwargs.get("start")
+        stop = kwargs.get("stop")
 
-        return PagingList(self._generate_drafts, 128, slice(start, stop))
+        paging_list = PagingList(self._generate_drafts, 128)
+        if start is not None or stop is not None:
+            return paging_list[start:stop]
+
+        return paging_list
 
     @Deprecated(since="1.2.0", removed_in="1.5.0", substitute=list_drafts)
     def list_draft_titles_and_numbers(
@@ -291,7 +297,8 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
             The :class:`.Commit` instance with the given revision.
 
         Raises:
-            TypeError: When the required commit does not exist or the given revision is illegal.
+            TypeError: When the given revision is illegal.
+            ResourceNotExistError: When the required commit does not exist.
 
         """
         if revision is None:
@@ -304,7 +311,7 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
         try:
             commit = next(self._generate_commits(revision))
         except StopIteration as error:
-            raise TypeError(f"The commit: {revision} does not exist.") from error
+            raise ResourceNotExistError(resource="commit", identification=revision) from error
 
         return commit
 
@@ -328,14 +335,16 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
             The PagingList of :class:`commits<.Commit>`.
 
         """
-        start = kwargs.get("start", 0)
-        stop = kwargs.get("stop", sys.maxsize)
+        start = kwargs.get("start")
+        stop = kwargs.get("stop")
 
-        return PagingList(
-            lambda offset, limit: self._generate_commits(revision, offset, limit),
-            128,
-            slice(start, stop),
+        paging_list = PagingList(
+            lambda offset, limit: self._generate_commits(revision, offset, limit), 128
         )
+        if start is not None or stop is not None:
+            return paging_list[start:stop]
+
+        return paging_list
 
     def create_tag(self, name: str, revision: Optional[str] = None) -> None:
         """Create the tag for a commit.
@@ -365,7 +374,8 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
             The :class:`.Tag` instance with the given name.
 
         Raises:
-            TypeError: When the required tag does not exist or the given tag is illegal.
+            TypeError: When the given tag is illegal.
+            ResourceNotExistError: When the required tag does not exist.
 
         """
         if not name:
@@ -374,7 +384,7 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
         try:
             tag = next(self._generate_tags(name))
         except StopIteration as error:
-            raise TypeError(f"The tag: {name} does not exist.") from error
+            raise ResourceNotExistError(resource="tag", identification=name) from error
 
         return tag
 
@@ -394,14 +404,16 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
             The PagingList of :class:`tags<.Tag>`.
 
         """
-        start = kwargs.get("start", 0)
-        stop = kwargs.get("stop", sys.maxsize)
+        start = kwargs.get("start")
+        stop = kwargs.get("stop")
 
-        return PagingList(
-            lambda offset, limit: self._generate_tags(None, offset, limit),
-            128,
-            slice(start, stop),
+        paging_list = PagingList(
+            lambda offset, limit: self._generate_tags(None, offset, limit), 128
         )
+        if start is not None or stop is not None:
+            return paging_list[start:stop]
+
+        return paging_list
 
     def get_branch(self, name: str) -> Branch:
         """Get the branch with the given name.
@@ -413,7 +425,8 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
             The :class:`.Branch` instance with the given name.
 
         Raises:
-            TypeError: When the required branch does not exist or the given branch is illegal.
+            TypeError: When the given branch is illegal.
+            ResourceNotExistError: When the required branch does not exist.
 
         """
         if not name:
@@ -422,7 +435,7 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
         try:
             branch = next(self._generate_branches(name))
         except StopIteration as error:
-            raise TypeError(f"The branch: {name} does not exist.") from error
+            raise ResourceNotExistError(resource="branch", identification=name) from error
 
         return branch
 
@@ -442,14 +455,16 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
             The PagingList of :class:`branches<.Branch>`.
 
         """
-        start = kwargs.get("start", 0)
-        stop = kwargs.get("stop", sys.maxsize)
-
-        return PagingList(
+        start = kwargs.get("start")
+        stop = kwargs.get("stop")
+        paging_list = PagingList(
             lambda offset, limit: self._generate_branches(None, offset, limit),
             128,
-            slice(start, stop),
         )
+        if start is not None or stop is not None:
+            return paging_list[start:stop]
+
+        return paging_list
 
     def checkout(self, revision: Optional[str] = None, draft_number: Optional[int] = None) -> None:
         """Checkout to commit or draft.
@@ -544,10 +559,14 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
             The PagingList of segment names.
 
         """
-        start = kwargs.get("start", 0)
-        stop = kwargs.get("stop", sys.maxsize)
+        start = kwargs.get("start")
+        stop = kwargs.get("stop")
 
-        return PagingList(self._generate_segment_names, 128, slice(start, stop))
+        paging_list = PagingList(self._generate_segment_names, 128)
+        if start is not None or stop is not None:
+            return paging_list[start:stop]
+
+        return paging_list
 
     def get_catalog(self) -> Catalog:
         """Get the catalog of the certain commit.
@@ -620,6 +639,23 @@ class DatasetClient(DatasetClientBase):
 
     """
 
+    def _generate_segments(
+        self, offset: int = 0, limit: int = 128
+    ) -> Generator[Segment, None, int]:
+        response = self._list_segments(offset, limit)
+
+        for item in response["segments"]:
+            segment = Segment._from_client(  # pylint: disable=protected-access
+                SegmentClient(item["name"], self)
+            )
+            segment.description = item["description"]
+            yield segment
+
+        return response["totalCount"]  # type: ignore[no-any-return]
+
+    def _list_segment_instances(self) -> PagingList[Segment]:
+        return PagingList(self._generate_segments, 128)
+
     def _upload_segment(
         self,
         segment: Segment,
@@ -671,14 +707,14 @@ class DatasetClient(DatasetClientBase):
             The created :class:`~tensorbay.client.segment.SegmentClient` with given name.
 
         Raises:
-            TypeError: When the segment exists.
+            NameConflictError: When the segment exists.
 
         """
         self._status.check_authority_for_draft()
         if name not in self.list_segment_names():
             self._create_segment(name)
         else:
-            raise TypeError(f"The segment: {name} exists")
+            raise NameConflictError(resource="segment", identification=name)
         return SegmentClient(name, self)
 
     def get_segment(self, name: str = "") -> SegmentClient:
@@ -691,11 +727,11 @@ class DatasetClient(DatasetClientBase):
             The required class:`~tensorbay.client.segment.SegmentClient`.
 
         Raises:
-            GASSegmentError: When the required segment does not exist.
+            ResourceNotExistError: When the required segment does not exist.
 
         """
         if name not in self.list_segment_names():
-            raise GASSegmentError(name)
+            raise ResourceNotExistError(resource="segment", identification=name)
 
         return SegmentClient(name, self)
 
@@ -747,19 +783,54 @@ class FusionDatasetClient(DatasetClientBase):
 
     """
 
+    @staticmethod
+    def _skip_upload_frame(
+        segment_filter: Iterator[Tuple[Frame, Optional[int], bool]],
+        done_set: Dict[float, Frame],
+        pbar: Tqdm,
+    ) -> Iterator[Tuple[Frame, Optional[int], bool]]:
+        for frame, timestamp, _ in segment_filter:
+            if timestamp is None:
+                timestamp = frame.frame_id.timestamp().timestamp
+            remote_frame = done_set.get(timestamp)
+            if remote_frame is None:
+                yield frame, timestamp, False
+            elif len(frame) != len(remote_frame):
+                remote_frame.frame_id = frame.frame_id
+                yield remote_frame, None, True
+            else:
+                pbar.update()
+
+    def _generate_segments(
+        self, offset: int = 0, limit: int = 128
+    ) -> Generator[FusionSegment, None, int]:
+        response = self._list_segments(offset, limit)
+
+        for item in response["segments"]:
+            segment = FusionSegment._from_client(  # pylint: disable=protected-access
+                FusionSegmentClient(item["name"], self)
+            )
+            segment.description = item["description"]
+            yield segment
+
+        return response["totalCount"]  # type: ignore[no-any-return]
+
+    def _list_segment_instances(self) -> PagingList[FusionSegment]:
+        return PagingList(self._generate_segments, 128)
+
     def _upload_segment(
         self,
         segment: FusionSegment,
         *,
         jobs: int,
-        skip_uploaded_files: bool,  # pylint: disable=unused-argument
+        skip_uploaded_files: bool,
         pbar: Tqdm,
     ) -> FusionSegmentClient:
         segment_client = self.get_or_create_segment(segment.name)
         for sensor in segment.sensors.values():
             segment_client.upload_sensor(sensor)
 
-        segment_filter: Iterator[Tuple[Frame, Optional[int]]]
+        segment_filter: Iterator[Tuple[Frame, Optional[int], bool]]
 
         if not segment:
             return segment_client
@@ -773,9 +844,18 @@ class FusionDatasetClient(DatasetClientBase):
                 )
 
         if have_frame_id:
-            segment_filter = ((frame, None) for frame in segment)
+            segment_filter = ((frame, None, False) for frame in segment)
         else:
-            segment_filter = ((frame, 10 * index + 10) for index, frame in enumerate(segment))
+            segment_filter = (
+                (frame, 10 * index + 10, False) for index, frame in enumerate(segment)
+            )
+
+        if skip_uploaded_files:
+            done_set = {
+                frame.frame_id.timestamp().timestamp: frame
+                for frame in segment_client.list_frames()
+            }
+            segment_filter = self._skip_upload_frame(segment_filter, done_set, pbar)
 
         multithread_upload(
             lambda args: segment_client.upload_frame(*args),
@@ -811,14 +891,14 @@ class FusionDatasetClient(DatasetClientBase):
             The created :class:`~tensorbay.client.segment.FusionSegmentClient` with given name.
 
         Raises:
-            TypeError: When the segment exists.
+            NameConflictError: When the segment exists.
 
         """
         self._status.check_authority_for_draft()
         if name not in self.list_segment_names():
             self._create_segment(name)
         else:
-            raise TypeError(f"The segment: {name} exists")
+            raise NameConflictError(resource="segment", identification=name)
         return FusionSegmentClient(name, self)
 
     def get_segment(self, name: str = "") -> FusionSegmentClient:
@@ -831,11 +911,11 @@ class FusionDatasetClient(DatasetClientBase):
             The required class:`~tensorbay.client.segment.FusionSegmentClient`.
 
         Raises:
-            GASSegmentError: When the required fusion segment does not exist.
+            ResourceNotExistError: When the required fusion segment does not exist.
 
         """
         if name not in self.list_segment_names():
-            raise GASSegmentError(name)
+            raise ResourceNotExistError(resource="segment", identification=name)
         return FusionSegmentClient(name, self)
 
     def upload_segment(
