@@ -15,17 +15,19 @@ import logging
 import os
 from collections import defaultdict
 from concurrent.futures import FIRST_EXCEPTION, ThreadPoolExecutor, wait
-from itertools import count
-from threading import Lock
+from itertools import repeat, zip_longest
 from typing import (
     Any,
     Callable,
     DefaultDict,
-    Dict,
     Generator,
+    Generic,
     Iterable,
+    Iterator,
+    List,
+    MutableSequence,
     Optional,
-    Sequence,
+    Tuple,
     TypeVar,
     Union,
     overload,
@@ -40,8 +42,8 @@ from tqdm import tqdm
 from urllib3.util.retry import Retry
 
 from ..__verison__ import __version__
-from ..exception import ResponseError
-from ..utility import ReprMixin, ReprType
+from ..exception import ResponseError, ResponseErrorDistributor
+from ..utility import ReprMixin, ReprType, locked
 from .log import RequestLogging, ResponseLogging
 
 logger = logging.getLogger(__name__)
@@ -136,7 +138,7 @@ class UserSession(Session):  # pylint: disable=too-few-public-methods
 
     def request(  # type: ignore[override]  # pylint: disable=signature-differs
         self, method: str, url: str, *args: Any, **kwargs: Any
-    ) -> Response:
+    ) -> Response:  # noqa: DAR401
         """Make the request.
 
         Arguments:
@@ -148,7 +150,7 @@ class UserSession(Session):  # pylint: disable=too-few-public-methods
         Returns:
             Response of the request.
 
-        Raises:
+        Raises:  # noqa: DAR402
             ResponseError: If post response error.
 
         """
@@ -158,7 +160,8 @@ class UserSession(Session):  # pylint: disable=too-few-public-methods
                 logger.error(
                     "Unexpected status code(%d)!%s", response.status_code, ResponseLogging(response)
                 )
-                raise ResponseError(response)
+                error_code = response.json()["code"]
+                raise ResponseErrorDistributor.get(error_code, ResponseError)(response)
             logger.debug(ResponseLogging(response))
             return response
 
@@ -339,46 +342,199 @@ def multithread_upload(
             future.result()
 
 
-class PagingList(Sequence[_T], ReprMixin):  # pylint: disable=too-many-ancestors
-    """PagingList is a wrap of web paging request.
+PagingGenerator = Callable[[int, int], Generator[_T, None, int]]
 
-    It follows the python Sequence protocal, which means it can be used like a python builtin list.
-    And it provides features like lazy evaluation and cache.
+
+class LazyItem(Generic[_T]):  # pylint: disable=too-few-public-methods
+    """In paging lazy evaluation system, a LazyItem instance represents an element in a pagination.
+
+    If user wants to access the elememt, LazyItem will trigger the paging request to pull a page of
+    elements and return the required element. All the pulled elements will be stored in different
+    LazyItem instances and will not be requested again.
 
     Arguments:
-        func: A paging generator function, which inputs offset<int> and limit<int> and returns a
-            generator. The returned generator should yield the element user needs, and return the
-            total count of the elements in the paging request.
+        page: The page the item belongs to.
+
+    Attributes:
+        page: The parent :class:`LazyPage` of this item.
+        data: The actual element stored in this item.
+
+    """
+
+    _S = TypeVar("_S", bound="LazyItem[_T]")
+
+    __slots__ = ("page", "data")
+
+    def __init__(self, page: "LazyPage[_T]", data: _T):
+        self.page = page
+        self.data = data
+
+    @classmethod
+    def from_page(cls, page: "LazyPage[_T]") -> "LazyItem[_T]":
+        """Create a LazyItem instance from page.
+
+        Arguments:
+            page: The page of the element.
+
+        Returns:
+            The LazyItem instance which stores the input page.
+
+        """
+        obj: "LazyItem[_T]" = object.__new__(cls)
+        obj.page = page
+        return obj
+
+    @classmethod
+    def from_data(cls, data: _T) -> "LazyItem[_T]":
+        """Create a LazyItem instance from data.
+
+        Arguments:
+            data: The actual data needs to be stored in LazyItem.
+
+        Returns:
+            The LazyItem instance which stores the input data.
+
+        """
+        obj: "LazyItem[_T]" = object.__new__(cls)
+        obj.data = data
+        return obj
+
+    def get(self) -> _T:
+        """Access the actual element represented by LazyItem.
+
+        If the element is already pulled from web, it will be return directly, otherwise this
+        function will request for a page of elements to get the required elememt.
+
+        Returns:
+            The actual element this LazyItem instance represents.
+
+        """
+        if not hasattr(self, "data"):
+            self.page.pull()
+
+        return self.data
+
+
+_R = TypeVar("_R")
+
+
+class ReturnGenerator(Generic[_T, _R]):  # pylint: disable=too-few-public-methods
+    """ReturnGenerator is a generator wrap to get the return value easily.
+
+    Arguments:
+        generator: The generator needs to be wrapped.
+
+    Attributes:
+        value: The return value of the input generator.
+
+    """
+
+    value: _R
+
+    def __init__(self, generator: Generator[_T, Any, _R]):
+        self._generator = generator
+
+    def __iter__(self) -> Iterator[_T]:
+        self.value = yield from self._generator
+
+
+class LazyPage(Generic[_T]):  # pylint: disable=too-few-public-methods
+    """In paging lazy evaluation system, a LazyPage instance represents a page with elements.
+
+    LazyPage is used for sending paging request to pull a page of elements and storing them in
+    different :class:`LazyItem` instances.
+
+    Arguments:
+        offset: The offset of the page.
+        limit: The limit of the page.
+        func: A paging generator function, which takes offset<int> and limit<int> as inputs and
+            returns a generator. The returned generator should yield the element user needs, and
+            return the total count of the elements in the paging request.
+
+    Attributes:
+        items: The :class:`LazyItem` list which represents a page of elements.
+
+    """
+
+    __slots__ = ("_offset", "_limit", "_func", "items")
+
+    def __init__(self, offset: int, limit: int, func: PagingGenerator[_T]) -> None:
+        self.items: Tuple[LazyItem[_T], ...] = tuple(LazyItem.from_page(self) for _ in range(limit))
+
+        self._init(offset, limit, func)
+
+    def _init(self, offset: int, limit: int, func: PagingGenerator[_T]) -> None:
+        self._offset = offset
+        self._limit = limit
+        self._func = func
+
+    @locked
+    def pull(self) -> None:
+        """Send paging request to pull a page of elements and store them in :class:`LazyItem`."""
+        for data, item in zip(self._func(self._offset, self._limit), self.items):
+            item.data = data
+
+
+class InitPage(LazyPage[_T]):  # pylint: disable=too-few-public-methods
+    """In paging lazy evaluation system, InitPage is the page for initializing :class:`PagingList`.
+
+    InitPage will send a paging request to pull a page of elements and storing them in different
+    :class:`LazyItem` instances when construction. And the totalCount of the page will also be
+    stored in the instance.
+
+    Arguments:
+        offset: The offset of the page.
+        limit: The limit of the page.
+        func: A paging generator function, which takes offset<int> and limit<int> as inputs and
+            returns a generator. The returned generator should yield the element user needs, and
+            return the total count of the elements in the paging request.
+
+    Attributes:
+        items: The :class:`LazyItem` list which represents a page of elements.
+        total_count: The totalCount of the paging request.
+
+    """
+
+    __slots__ = LazyPage.__slots__ + ("total_count",)
+
+    def __init__(  # pylint: disable=super-init-not-called
+        self, offset: int, limit: int, func: PagingGenerator[_T]
+    ) -> None:
+        generator = ReturnGenerator(func(offset, limit))
+        self.items: Tuple[LazyItem[_T], ...] = tuple(LazyItem(self, data) for data in generator)
+
+        self._init(offset, len(self.items), func)
+
+        self.total_count = generator.value
+
+
+class PagingList(MutableSequence[_T], ReprMixin):  # pylint: disable=too-many-ancestors
+    """PagingList is a wrap of web paging request.
+
+    It follows the python MutableSequence protocal, which means it can be used like a python builtin
+    list. And it provides features like lazy evaluation and cache.
+
+    Arguments:
+        func: A paging generator function, which takes offset<int> and limit<int> as inputs and
+            returns a generator. The returned generator should yield the element user needs, and
+            return the total count of the elements in the paging request.
         limit: The page size of each paging request.
-        slicing: The required slice of PagingList.
 
     """
 
     _S = TypeVar("_S", bound="PagingList[_T]")
-    _K = TypeVar("_K")
 
     _repr_type = ReprType.SEQUENCE
 
-    def __init__(
-        self,
-        func: Callable[[int, int], Generator[_T, None, int]],
-        limit: int,
-        slicing: slice = slice(0, None),
-    ) -> None:
-        self._data: Dict[int, _T] = {}
-        self._attr: Dict[str, int] = {}
-        self._locks: DefaultDict[Union[int, str], Lock] = defaultdict(Lock)
+    _items: List[LazyItem[_T]]
+
+    def __init__(self, func: PagingGenerator[_T], limit: int) -> None:
         self._func = func
         self._limit = limit
-
-        if slicing.step is not None:
-            raise ValueError("The step slice is not supported yet.")
-
-        self._slice = slicing
-        self._len: Optional[int] = None
+        self._init_items: Callable[[int], None] = self._init_all_items
 
     def __len__(self) -> int:
-        return self._get_len()
+        return self._get_items().__len__()
 
     @overload
     def __getitem__(self: _S, index: int) -> _T:
@@ -392,76 +548,92 @@ class PagingList(Sequence[_T], ReprMixin):  # pylint: disable=too-many-ancestors
         if isinstance(index, slice):
             return self._get_slice(index)
 
-        index = self._make_index_positive(index)
-        if index < 0 or index >= self._get_len(index):
-            raise IndexError("list index out of range")
+        return self._get_items(index)[index].get()
 
-        paging_index = self._slice.start + index
-        if paging_index not in self._data:
-            offset = self._get_offset(paging_index)
-            self._call_with_lock(offset, self._extend, offset)
+    @overload
+    def __setitem__(self, index: int, value: _T) -> None:
+        ...
 
-        return self._data[paging_index]
+    @overload
+    def __setitem__(self, index: slice, value: Iterable[_T]) -> None:
+        ...
 
-    def _extend(self, offset: int) -> int:
-        generator = self._func(offset, self._limit)
-        try:
-            for i in count(offset):
-                data = next(generator)
-                self._data[i] = data
-        except StopIteration as error:
-            return error.value  # type: ignore[no-any-return]
+    def __setitem__(self, index: Union[int, slice], value: Union[_T, Iterable[_T]]) -> None:
+        # https://github.com/python/mypy/issues/7858
+        if isinstance(index, slice):
+            self._get_items().__setitem__(
+                index,
+                map(LazyItem.from_data, value),  # type: ignore[arg-type]
+            )
+            return
 
-        raise TypeError("Impossible to be here, add this to make pylint and mypy happy")
+        self._get_items(index).__setitem__(
+            index,
+            LazyItem.from_data(value),  # type: ignore[arg-type]
+        )
 
-    def _get_total_count(self, index: int) -> None:
-        offset = self._get_offset(index)
-        self._attr["totalCount"] = self._extend(offset)
+    def __delitem__(self, index: Union[int, slice]) -> None:
+        self._get_items().__delitem__(index)
 
-    def _get_len(self, index: int = 0) -> int:
-        if self._len is None:
-            if "totalCount" not in self._attr:
-                self._call_with_lock("totalCount", self._get_total_count, index)
+    @staticmethod
+    def _range(total_count: int, limit: int) -> Iterator[Tuple[int, int]]:
+        """A Generator which generates offset and limit for paging request.
 
-            total_count = self._attr["totalCount"]
-            stop = total_count if self._slice.stop is None else min(total_count, self._slice.stop)
-            start = min(total_count, self._slice.start)
-            self._len = stop - start
+        Examples:
+            >>> self._range(10, 3)
+            <generator object paging_range at 0x11b9932e0>
 
-        return self._len
+            >>> list(self._range(10, 3))
+            [(0, 3), (3, 3), (6, 3), (9, 1)]
 
-    def _make_index_positive(self, index: int) -> int:
-        return index if index >= 0 else self._get_len() + index
+        Arguments:
+            total_count: The total count of the page.
+            limit: The paging limit.
 
-    def _get_offset(self, index: int) -> int:
-        return index // self._limit * self._limit
+        Yields:
+            The tuple (offset, limit) for paging request.
 
-    def _call_with_lock(self, key: Union[int, str], func: Callable[[_K], Any], arg: _K) -> None:
-        lock = self._locks[key]
-        acquire = lock.acquire(blocking=False)
-        try:
-            if acquire:
-                func(arg)
-                del self._locks[key]
-            else:
-                lock.acquire()
-        finally:
-            lock.release()
+        """
+        div, mod = divmod(total_count, limit)
+        yield from zip_longest(range(0, total_count, limit), repeat(limit, div), fillvalue=mod)
 
-    def _get_slice(self: _S, input_slice: slice) -> _S:
-        start = self._slice.start
-        if input_slice.start is not None:
-            start += self._make_index_positive(input_slice.start)
+    @locked
+    def _init_all_items(self, index: int = 0) -> None:
+        index = index if index >= 0 else 0
+        index_offset = index // self._limit * self._limit
+        init_page = InitPage(index_offset, self._limit, self._func)
+        total_count = init_page.total_count
+        self._items: List[LazyItem[_T]] = []
+        for offset, limit in self._range(total_count, self._limit):
+            page = init_page if offset == index_offset else LazyPage(offset, limit, self._func)
+            self._items.extend(page.items)
 
-        stop = input_slice.stop
-        if stop is not None:
-            stop = self._make_index_positive(stop) + self._slice.start
+    @locked
+    def _init_sliced_items(self, parent: _S, slicing: slice) -> None:
+        self._items = parent._get_items()[slicing]  # pylint: disable=protected-access
 
-        paging_list = self.__class__(self._func, self._limit, slice(start, stop))
+    def _get_items(self, index: int = 0) -> List[LazyItem[_T]]:
+        if not hasattr(self, "_items"):
+            self._init_items(index)
 
-        # The sliced PagingList shares the "_data", "_attr" and "_locks" with the root PagingList
-        paging_list._data = self._data  # pylint: disable=protected-access
-        paging_list._attr = self._attr  # pylint: disable=protected-access
-        paging_list._locks = self._locks  # pylint: disable=protected-access
+        return self._items
+
+    def _get_slice(self: _S, slicing: slice) -> _S:
+        # pylint: disable=protected-access
+        paging_list = self.__class__(self._func, self._limit)
+        if hasattr(self, "_items"):
+            paging_list._items = self._items[slicing]
+        else:
+            paging_list._init_items = lambda index: paging_list._init_sliced_items(self, slicing)
 
         return paging_list
+
+    def insert(self, index: int, value: _T) -> None:
+        """Insert object before index.
+
+        Arguments:
+            index: Position of the PagingList.
+            value: Element to be inserted into the PagingList.
+
+        """
+        self._get_items(index).insert(index, LazyItem.from_data(value))
