@@ -23,12 +23,37 @@ It consists of a list of :class:`~tensorbay.dataset.segment.FusionSegment`.
 """
 
 import json
-from typing import Any, Dict, KeysView, Sequence, Type, TypeVar, Union, overload
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Iterable,
+    KeysView,
+    Optional,
+    Sequence,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    overload,
+)
 
 from ..label import Catalog
-from ..utility import EqMixin, NameMixin, NameSortedList, ReprMixin, ReprType, common_loads
+from ..utility import (
+    Deprecated,
+    EqMixin,
+    NameMixin,
+    NameSortedList,
+    ReprMixin,
+    ReprType,
+    common_loads,
+    locked,
+)
 from .segment import FusionSegment, Segment
 
+if TYPE_CHECKING:
+    from ..client import GAS
+
 _T = TypeVar("_T", FusionSegment, Segment)
 
 
@@ -37,15 +62,19 @@ class Notes(ReprMixin, EqMixin):
 
     Arguments:
         is_continuous: Whether the data inside the dataset is time-continuous.
+        bin_point_cloud_fields: The field names of the bin point cloud files in the dataset.
 
     """
 
     _T = TypeVar("_T", bound="Notes")
 
-    _repr_attrs = ("is_continuous",)
+    _repr_attrs = ("is_continuous", "bin_point_cloud_fields")
 
-    def __init__(self, is_continuous: bool = False) -> None:
+    def __init__(
+        self, is_continuous: bool = False, bin_point_cloud_fields: Optional[Iterable[str]] = None
+    ) -> None:
         self.is_continuous = is_continuous
+        self.bin_point_cloud_fields = bin_point_cloud_fields
 
     def __getitem__(self, key: str) -> Any:
         try:
@@ -55,6 +84,7 @@ class Notes(ReprMixin, EqMixin):
 
     def _loads(self, contents: Dict[str, Any]) -> None:
         self.is_continuous = contents["isContinuous"]
+        self.bin_point_cloud_fields = contents.get("binPointCloudFields")
 
     @classmethod
     def loads(cls: Type[_T], contents: Dict[str, Any]) -> _T:
@@ -64,7 +94,11 @@ class Notes(ReprMixin, EqMixin):
             contents: The given dict containing the dataset notes::
 
                     {
-                        "isContinuous": <boolean>
+                        "isContinuous":            <boolean>
+                        "binPointCloudFields": [   <array> or null
+                                <field_name>,      <str>
+                                ...
+                        ]
                     }
 
         Returns:
@@ -80,11 +114,7 @@ class Notes(ReprMixin, EqMixin):
             The valid keys within the notes.
 
         """
-        keys = set()
-        for attr in self._repr_attrs:
-            if hasattr(self, attr):
-                keys.add(attr)
-        return KeysView(keys)  # type: ignore[arg-type]
+        return KeysView(self._repr_attrs)  # type: ignore[arg-type]
 
     def dumps(self) -> Dict[str, Any]:
         """Dumps the notes into a dict.
@@ -93,11 +123,18 @@ class Notes(ReprMixin, EqMixin):
             A dict containing all the information of the Notes::
 
                 {
-                    "isContinuous": <boolean>
+                    "isContinuous":           <boolean>
+                    "binPointCloudFields": [  <array> or null
+                        <field_name>,         <str>
+                        ...
+                    ]
                 }
 
         """
-        return {"isContinuous": self.is_continuous}
+        contents: Dict[str, Any] = {"isContinuous": self.is_continuous}
+        if self.bin_point_cloud_fields:
+            contents["binPointCloudFields"] = self.bin_point_cloud_fields
+        return contents
 
 
 class DatasetBase(NameMixin, Sequence[_T]):  # pylint: disable=too-many-ancestors
@@ -111,30 +148,62 @@ class DatasetBase(NameMixin, Sequence[_T]):  # pylint: disable=too-many-ancestor
 
     Arguments:
         name: The name of the dataset.
+        gas: The :class:`~tensorbay.client.gas.GAS` client for getting a remote dataset.
+        revision: The revision of the remote dataset.
+
+    Attributes:
+        catalog: The :class:`~tensorbay.label.catalog.Catalog` of the dataset.
+        notes: The :class:`Notes` of the dataset.
 
     """
 
+    _is_fusion: bool
+
     _repr_type = ReprType.SEQUENCE
 
-    def __init__(self, name: str) -> None:
+    def __init__(
+        self, name: str, gas: Optional["GAS"] = None, revision: Optional[str] = None
+    ) -> None:
         super().__init__(name)
-        self._segments: NameSortedList[_T] = NameSortedList()
-        self._catalog: Catalog = Catalog()
-        self._notes = Notes()
+
+        if gas:
+            self._client = gas.get_dataset(name, is_fusion=self._is_fusion)
+            if revision:
+                self._client.checkout(revision)
+        else:
+            self._segments: NameSortedList[_T] = NameSortedList()
+            self._catalog = Catalog()
+            self._notes = Notes()
 
     def __len__(self) -> int:
-        return self._segments.__len__()
+        return self._get_segments().__len__()
 
     @overload
-    def __getitem__(self, index: int) -> _T:
+    def __getitem__(self, index: Union[int, str]) -> _T:
         ...
 
     @overload
     def __getitem__(self, index: slice) -> Sequence[_T]:
         ...
 
-    def __getitem__(self, index: Union[int, slice]) -> Union[Sequence[_T], _T]:
-        return self._segments.__getitem__(index)
+    def __getitem__(self, index: Union[int, str, slice]) -> Union[Sequence[_T], _T]:
+        if isinstance(index, str):
+            return self._get_segments().get_from_name(index)
+
+        return self._get_segments().__getitem__(index)
+
+    @locked
+    def _init_segments(self) -> None:
+        self._segments = NameSortedList()
+        # pylint: disable=protected-access
+        for segment in self._client._list_segment_instances():
+            self._segments.add(segment)  # type: ignore[arg-type]
+
+    def _get_segments(self) -> NameSortedList[_T]:
+        if not hasattr(self, "_segments"):
+            self._init_segments()
+
+        return self._segments
 
     @property
     def catalog(self) -> Catalog:
@@ -144,6 +213,9 @@ class DatasetBase(NameMixin, Sequence[_T]):  # pylint: disable=too-many-ancestor
             The :class:`~tensorbay.label.catalog.Catalog` of the dataset.
 
         """
+        if not hasattr(self, "_catalog"):
+            self._catalog = self._client.get_catalog()
+
         return self._catalog
 
     @property
@@ -154,8 +226,21 @@ class DatasetBase(NameMixin, Sequence[_T]):  # pylint: disable=too-many-ancestor
             The class:`Notes` of the dataset.
 
         """
+        if not hasattr(self, "_notes"):
+            self._notes = self._client.get_notes()
+
         return self._notes
 
+    def keys(self) -> Tuple[str, ...]:
+        """Get all segment names.
+
+        Returns:
+            A tuple containing all segment names.
+
+        """
+        # pylint: disable=protected-access
+        return tuple(self._segments._data)
+
     def load_catalog(self, filepath: str) -> None:
         """Load catalog from a json file.
 
@@ -167,6 +252,7 @@ class DatasetBase(NameMixin, Sequence[_T]):  # pylint: disable=too-many-ancestor
             contents = json.load(fp)
         self._catalog = Catalog.loads(contents)
 
+    @Deprecated(since="v1.4.0", removed_in="v1.7.0", substitute=__getitem__)
     def get_segment_by_name(self, name: str) -> _T:
         """Return the segment corresponding to the given name.
 
@@ -177,7 +263,7 @@ class DatasetBase(NameMixin, Sequence[_T]):  # pylint: disable=too-many-ancestor
             The segment which matches the input name.
 
         """
-        return self._segments.get_from_name(name)
+        return self._get_segments().get_from_name(name)
 
     def add_segment(self, segment: _T) -> None:
         """Add a segment to the dataset.
@@ -186,7 +272,7 @@ class DatasetBase(NameMixin, Sequence[_T]):  # pylint: disable=too-many-ancestor
             segment: The segment to be added.
 
         """
-        self._segments.add(segment)
+        self._get_segments().add(segment)
 
 
 class Dataset(DatasetBase[Segment]):
@@ -197,6 +283,8 @@ class Dataset(DatasetBase[Segment]):
 
     """
 
+    _is_fusion = False
+
     def create_segment(self, segment_name: str = "") -> Segment:
         """Create a segment with the given name.
 
@@ -208,7 +296,7 @@ class Dataset(DatasetBase[Segment]):
 
         """
         segment = Segment(segment_name)
-        self._segments.add(segment)
+        self._get_segments().add(segment)
         return segment
 
 
@@ -219,6 +307,8 @@ class FusionDataset(DatasetBase[FusionSegment]):
     It consists of a list of :class:`~tensorbay.dataset.segment.FusionSegment`.
     """
 
+    _is_fusion = True
+
     def create_segment(self, segment_name: str = "") -> FusionSegment:
         """Create a fusion segment with the given name.
 
@@ -231,5 +321,5 @@ class FusionDataset(DatasetBase[FusionSegment]):
 
         """
         segment = FusionSegment(segment_name)
-        self._segments.add(segment)
+        self._get_segments().add(segment)
         return segment
