@@ -13,11 +13,17 @@ from different sensors.
 
 """
 
+import logging
 from typing import Any, Dict, Optional, Type, TypeVar
+from uuid import UUID
+
+from ulid import ULID, from_str, from_uuid
 
 from ..utility import UserMutableMapping, common_loads
 from .data import DataBase
 
+logger = logging.getLogger(__name__)
+
 
 class Frame(UserMutableMapping[str, "DataBase._Type"]):
     """This class defines the concept of frame.
@@ -41,17 +47,36 @@ class Frame(UserMutableMapping[str, "DataBase._Type"]):
     """
 
     _T = TypeVar("_T", bound="Frame")
+    _logger_flag = True
 
-    def __init__(self, frame_id: Optional[str] = None) -> None:
+    def __init__(self, frame_id: Optional[ULID] = None) -> None:
         self._data: Dict[str, DataBase._Type] = {}
         # self._pose: Optional[Transform3D] = None
         if frame_id:
             self.frame_id = frame_id
 
+    def _repr_head(self) -> str:
+        if hasattr(self, "frame_id"):
+            return f'{self.__class__.__name__}("{self.frame_id}")'
+
+        return self.__class__.__name__
+
     def _loads(self, contents: Dict[str, Any]) -> None:
         self._data = {}
         if "frameId" in contents:
-            self.frame_id = contents["frameId"]
+            try:
+                self.frame_id = from_str(contents["frameId"])
+            except ValueError:
+                # Legacy fusion dataset use uuid as frame ID
+                # Keep this code here to make SDK compatible with uuid
+                self.frame_id = from_uuid(UUID(contents["frameId"]))
+                if self.__class__._logger_flag:  # pylint: disable=protected-access
+                    self.__class__._logger_flag = False  # pylint: disable=protected-access
+                    logger.warning(
+                        "WARNING: This is a legacy fusion dataset which use uuid as frame ID, "
+                        "it should be updated to ulid."
+                    )
+
         for data_contents in contents["frame"]:
             self._data[data_contents["sensorName"]] = DataBase.loads(data_contents)
 
@@ -107,7 +132,7 @@ class Frame(UserMutableMapping[str, "DataBase._Type"]):
         # if self._pose:
         #     contents["pose"] = self._pose.dumps()
         if hasattr(self, "frame_id"):
-            contents["frameId"] = self.frame_id
+            contents["frameId"] = str(self.frame_id)
 
         frame = []
         for sensor_name, data in self._data.items():
@@ -121,21 +146,19 @@ class Frame(UserMutableMapping[str, "DataBase._Type"]):
 
     # def set_pose(
     #     self,
-    #     transform: Transform3D.TransformType = None,
-    #     *,
     #     translation: Optional[Iterable[float]] = (0, 0, 0),
     #     rotation: Transform3D.RotationType = (1, 0, 0, 0),
+    #     *,
+    #     matrix: Optional[MatrixType] = None,
     # ) -> None:
     #     """Set the pose of the current frame.
 
     #     Arguments:
-    #         transform: The transform representing the frame pose in
-    #             a :class:`~tensorbay.geometry.transform.Transform3D` object
-    #             or a 4x4 or 3x4 transformation matrix.
     #         translation: Translation of the frame pose in a sequence of [x, y, z].
     #         rotation: Rotation of the frame pose in a sequence of [w, x, y, z]
-    #             or a 3x3 rotation matrix
     #             or a numpy quaternion object.
+    #         matrix: The transform representing the frame pose in
+    #             a 4x4 or 3x4 transform matrix.
 
     #     """
-    #     self._pose = Transform3D(transform, translation=translation, rotation=rotation)
+    #     self._pose = Transform3D(translation, rotation, matrix=matrix)
