@@ -105,6 +105,7 @@ class KeypointsInfo(ReprMixin, EqMixin):
         description: The description of the keypoints.
 
     Attributes:
+        number: The number of the set of keypoints.
         names: All the names of the keypoints.
         skeleton: The skeleton of the keypoints
             indicating which keypoint should connect with another.
@@ -154,7 +155,7 @@ class KeypointsInfo(ReprMixin, EqMixin):
         skeleton: Optional[Iterable[Iterable[int]]] = None,
         visible: Optional[str] = None,
         parent_categories: Union[None, str, Iterable[str]] = None,
-        description: Optional[str] = None,
+        description: str = "",
     ):
         self._number = number
         if names:
@@ -169,8 +170,7 @@ class KeypointsInfo(ReprMixin, EqMixin):
             except KeyError as error:
                 raise ValueError("Visible can only be 'BINARY' or 'TERNARY'") from error
 
-        if description:
-            self.description = description
+        self.description = description
 
         if not parent_categories:
             return
@@ -195,8 +195,7 @@ class KeypointsInfo(ReprMixin, EqMixin):
         if "parentCategories" in contents:
             self.parent_categories = contents["parentCategories"]
 
-        if "description" in contents:
-            self.description = contents["description"]
+        self.description = contents.get("description", "")
 
     @classmethod
     def loads(cls: Type[_T], contents: Dict[str, Any]) -> _T:
@@ -357,7 +356,31 @@ class CategoriesMixin(SubcatalogMixin):  # pylint: disable=too-few-public-method
             contents["categoryDelimiter"] = self.category_delimiter
         return contents
 
-    def add_category(self, name: str, description: Optional[str] = None) -> None:
+    def get_category_to_index(self) -> Dict[str, int]:
+        """Return the dict containing the conversion from category to index.
+
+        Returns:
+            A dict containing the conversion from category to index.
+
+        """
+        if not hasattr(self, "categories"):
+            return {}
+
+        return {category: index for index, category in enumerate(self.categories)}
+
+    def get_index_to_category(self) -> Dict[int, str]:
+        """Return the dict containing the conversion from index to category.
+
+        Returns:
+            A dict containing the conversion from index to category.
+
+        """
+        if not hasattr(self, "categories"):
+            return {}
+
+        return dict(enumerate(self.categories))
+
+    def add_category(self, name: str, description: str = "") -> None:
         """Add a category to the Subcatalog.
 
         Arguments:
@@ -407,7 +430,7 @@ class AttributesMixin(SubcatalogMixin):  # pylint: disable=too-few-public-method
         maximum: Optional[float] = None,
         items: Optional[Items] = None,
         parent_categories: Union[None, str, Iterable[str]] = None,
-        description: Optional[str] = None,
+        description: str = "",
     ) -> None:
         """Add an attribute to the Subcatalog.
 
