@@ -3,8 +3,7 @@
 # Copyright 2021 Graviti. Licensed under MIT License.
 #
 # pylint: disable=invalid-name
-
-"""Dataloader of the LeedsSportsPose dataset."""
+# pylint: disable=missing-module-docstring
 
 import os
 
@@ -13,25 +12,28 @@ from ...geometry import Keypoint2D
 from ...label import LabeledKeypoints2D
 from .._utility import glob
 
-DATASET_NAME = "Leeds Sports Pose"
+DATASET_NAME = "LeedsSportsPose"
 
 
 def LeedsSportsPose(path: str) -> Dataset:
-    """Dataloader of the LeedsSportsPose dataset.
+    """Dataloader of the `Leeds Sports Pose`_ dataset.
+
+    .. _Leeds Sports Pose: https://sam.johnson.io/research/lsp.html
+
+    The folder structure should be like::
+
+        <path>
+            joints.mat
+            images/
+                im0001.jpg
+                im0002.jpg
+                ...
 
     Arguments:
         path: The root directory of the dataset.
-            The folder structure should be like::
-
-                <path>
-                    joints.mat
-                    images/
-                        im0001.jpg
-                        im0002.jpg
-                        ...
 
     Returns:
-        Loaded `Dataset` object.
+        Loaded :class:`~tensorbay.dataset.dataset.Dataset` instance.
 
     """
     from scipy.io import loadmat  # pylint: disable=import-outside-toplevel
