@@ -7,11 +7,13 @@
 
 from .AnimalPose import AnimalPose5, AnimalPose7
 from .AnimalsWithAttributes2 import AnimalsWithAttributes2
+from .BioIDFace import BioIDFace
 from .BSTLD import BSTLD
 from .CADC import CADC
 from .CarConnection import CarConnection
 from .CoinImage import CoinImage
 from .CompCars import CompCars
+from .COVIDChestXRay import COVIDChestXRay
 from .DeepRoute import DeepRoute
 from .DogsVsCats import DogsVsCats
 from .DownsampledImagenet import DownsampledImagenet
@@ -19,6 +21,7 @@ from .Elpv import Elpv
 from .FLIC import FLIC
 from .Flower import Flower17, Flower102
 from .FSDD import FSDD
+from .HalpeFullBody import HalpeFullBody
 from .HardHatWorkers import HardHatWorkers
 from .HeadPoseImage import HeadPoseImage
 from .ImageEmotion import ImageEmotionAbstract, ImageEmotionArtphoto
@@ -30,6 +33,7 @@ from .LISATrafficLight import LISATrafficLight
 from .NeolixOD import NeolixOD
 from .Newsgroups20 import Newsgroups20
 from .NightOwls import NightOwls
+from .nuScenes import nuScenes
 from .RP2K import RP2K
 from .THCHS30 import THCHS30
 from .THUCNews import THUCNews
@@ -37,13 +41,16 @@ from .TLR import TLR
 from .WIDER_FACE import WIDER_FACE
 
 __all__ = [
+    "nuScenes",
     "AnimalPose5",
     "AnimalPose7",
     "AnimalsWithAttributes2",
+    "BioIDFace",
     "BSTLD",
     "CADC",
     "CarConnection",
     "CoinImage",
+    "COVIDChestXRay",
     "CompCars",
     "DeepRoute",
     "DogsVsCats",
@@ -51,6 +58,7 @@ __all__ = [
     "Elpv",
     "FLIC",
     "FSDD",
+    "HalpeFullBody",
     "Flower102",
     "Flower17",
     "HardHatWorkers",
