@@ -9,10 +9,10 @@ import pytest
 import ulid
 
 from tensorbay import __version__
-from tensorbay.client.exceptions import GASResponseError
 from tensorbay.client.gas import GAS
 from tensorbay.dataset.data import Data, Label
 from tensorbay.dataset.frame import Frame
+from tensorbay.exception import FrameError, ResponseError
 from tensorbay.label.catalog import Catalog
 from tensorbay.sensor.sensor import Sensor, Sensors
 
@@ -115,7 +115,7 @@ class TestSegmentClient:
             local_path.write_text("CONTENT")
             segment_client.upload_file(local_path=str(local_path))
 
-        data = list(segment_client.list_data())
+        data = segment_client.list_data()
         assert data[0].path == "hello0.txt"
         assert data[0].open().read() == b"CONTENT"
         assert not data[0].label
@@ -142,7 +142,7 @@ class TestSegmentClient:
             local_path.write_text("ADD CONTENT")
             segment_client.upload_file(local_path=str(local_path))
 
-        data = list(segment_client.list_data())
+        data = segment_client.list_data()
         assert data[0].path == "hello0.txt"
         assert data[0].open().read() == b"ADD CONTENT"
         assert not data[0].label
@@ -169,7 +169,7 @@ class TestSegmentClient:
             local_path.write_text("CONTENT")
             segment_client.upload_file(local_path=str(local_path))
 
-        data = list(segment_client.list_data())
+        data = segment_client.list_data()
         assert data[0].path == "goodbye0.txt"
         assert data[5].path == "hello0.txt"
 
@@ -196,7 +196,7 @@ class TestSegmentClient:
             local_path.write_text("CONTENT")
             segment_client.upload_file(local_path=str(local_path))
 
-        data = list(segment_client.list_data())
+        data = segment_client.list_data()
         assert data[0].path == "goodbye0.txt"
         assert data[5].path == "hello0.txt"
 
@@ -222,7 +222,7 @@ class TestSegmentClient:
             local_path.write_text("CONTENT")
             segment_client.upload_file(local_path=str(local_path))
 
-        data_paths = list(segment_client.list_data_paths())
+        data_paths = segment_client.list_data_paths()
         assert data_paths[0] == "goodbye0.txt"
         assert data_paths[5] == "hello0.txt"
 
@@ -244,7 +244,7 @@ class TestSegmentClient:
         data = Data(local_path=str(local_path))
         data.label = Label.loads(LABEL)
         # If not uploading file, uploading label is not allowed
-        with pytest.raises(GASResponseError):
+        with pytest.raises(ResponseError):
             segment_client.upload_label(data)
 
         # Uploading files
@@ -252,12 +252,12 @@ class TestSegmentClient:
 
         data.label = Label.loads(WRONG_LABEL)
         # Uploading wrong label is not allowed
-        with pytest.raises(GASResponseError):
+        with pytest.raises(ResponseError):
             segment_client.upload_label(data)
         data.label = Label.loads(LABEL)
         segment_client.upload_label(data)
 
-        data = list(segment_client.list_data())
+        data = segment_client.list_data()
         assert data[0].path == "hello0.txt"
         assert data[0].label
         # todo: match the input and output label
@@ -285,7 +285,7 @@ class TestSegmentClient:
         data.label = Label.loads(NEW_LABEL)
         segment_client.upload_label(data)
 
-        data = list(segment_client.list_data())
+        data = segment_client.list_data()
         assert data[0].path == "hello0.txt"
         assert data[0].label
         # todo: match the input and output label
@@ -307,7 +307,7 @@ class TestSegmentClient:
 
         # If not uploading catalog, uploading label is not allowed
         data.label = Label.loads(LABEL)
-        with pytest.raises(GASResponseError):
+        with pytest.raises(ResponseError):
             segment_client.upload_label(data)
 
         gas_client.delete_dataset(dataset_name)
@@ -326,7 +326,7 @@ class TestSegmentClient:
             local_path.write_text("CONTENT")
             segment_client.upload_data(Data(local_path=str(local_path)))
 
-        data = list(segment_client.list_data())
+        data = segment_client.list_data()
         assert data[0].path == "hello0.txt"
         assert data[0].open().read() == b"CONTENT"
         assert not data[0].label
@@ -351,7 +351,7 @@ class TestSegmentClient:
             data.label = Label.loads(LABEL)
             segment_client.upload_data(data)
 
-        data = list(segment_client.list_data())
+        data = segment_client.list_data()
         assert data[0].path == "hello0.txt"
         assert data[0].open().read() == b"CONTENT"
         assert data[0].label
@@ -377,11 +377,11 @@ class TestSegmentClient:
             segment_client.upload_data(data)
 
         segment_client.delete_data("hello0.txt")
-        data_paths = list(segment_client.list_data_paths())
+        data_paths = segment_client.list_data_paths()
         assert "hello0.txt" not in data_paths
 
         segment_client.delete_data(segment_client.list_data_paths())
-        data = list(segment_client.list_data())
+        data = segment_client.list_data()
         assert len(data) == 0
 
         gas_client.delete_dataset(dataset_name)
@@ -423,7 +423,7 @@ class TestSegmentClient:
             frame[LIDAR_DATA["name"]] = data
             segment_client.upload_frame(frame, timestamp=i)
 
-        frames = list(segment_client.list_frames())
+        frames = segment_client.list_frames()
         assert len(frames) == 5
         assert frames[0][LIDAR_DATA["name"]].path == "hello0.txt"
         assert not frames[0][LIDAR_DATA["name"]].label
@@ -447,7 +447,7 @@ class TestSegmentClient:
         frame[LIDAR_DATA["name"]] = data
 
         # If not uploading sensor, uploading frame is not allowed
-        with pytest.raises(GASResponseError):
+        with pytest.raises(ResponseError):
             segment_client.upload_frame(frame, timestamp=0)
 
         gas_client.delete_dataset(dataset_name)
@@ -473,7 +473,7 @@ class TestSegmentClient:
             frame[LIDAR_DATA["name"]] = data
             segment_client.upload_frame(frame, timestamp=i)
 
-        frames = list(segment_client.list_frames())
+        frames = segment_client.list_frames()
         assert len(frames) == 5
         assert frames[0][LIDAR_DATA["name"]].path == "hello0.txt"
         assert frames[0][LIDAR_DATA["name"]].label
@@ -517,7 +517,7 @@ class TestSegmentClient:
         local_path.write_text("CONTENT")
         data = Data(local_path=str(local_path))
         frame[LIDAR_DATA["name"]] = data
-        with pytest.raises(TypeError):
+        with pytest.raises(FrameError):
             segment_client.upload_frame(frame, timestamp=i)
 
         # Neither setting frame id in frame nor set timestamp(order) when uploading is not allowed
@@ -526,10 +526,10 @@ class TestSegmentClient:
         local_path.write_text("CONTENT")
         data = Data(local_path=str(local_path))
         frame[LIDAR_DATA["name"]] = data
-        with pytest.raises(TypeError):
+        with pytest.raises(FrameError):
             segment_client.upload_frame(frame)
 
-        frames = list(segment_client.list_frames())
+        frames = segment_client.list_frames()
         assert len(frames) == 10
         assert frames[0][LIDAR_DATA["name"]].path == "hello0.txt"
         assert frames[5][LIDAR_DATA["name"]].path == "goodbye5.txt"
